@@ -35,7 +35,7 @@ import re
 
 PY2 = sys.version_info[0] == 2
 
-VERSION = (0, 1, 1)
+VERSION = (0, 2, 0)
 __version__ = '%s.%s.%s' % VERSION
 apilevel = '2.0'
 threadsafety = 1
@@ -423,27 +423,8 @@ class Connection(object):
         self.encoding = 'UTF8'
         self.autocommit = False
         self.in_transaction = False
+        self._open()
 
-        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
-        self.sock.connect((self.host, self.port))
-        DEBUG_OUTPUT("socket %s:%d" % (self.host, self.port))
-        if self.use_ssl:
-            self._write(_bint_to_bytes(8, 4))
-            self._write(_bint_to_bytes(80877103, 4))    # SSL request
-            if self_read(1) == b'S':
-                self.sock = ssl.wrap_socket(self.sock)
-            else:
-                raise InterfaceError("Server refuses SSL")
-        self.sock.settimeout(self.timeout)
-        v = b''.join([
-            _bint_to_bytes(196608, 4),  # protocol version 3.0
-            b'user\x00', user.encode('ascii'), b'\x00',
-            b'database\x00', database.encode('ascii'), b'\x00',
-            b'\x00',
-        ])
-        self._write(_bint_to_bytes(len(v) + 4, 4) + v)
-        self._cursor = self.cursor()
-        self._process_messages()
 
     def _send_message(self, code, data):
         self._write(
@@ -543,9 +524,30 @@ class Connection(object):
                     raise IntegrityError(errcode, message)
                 else:
                     raise ProgrammingError(errcode, message)
+            elif code == PG_B_COPY_OUT_RESPONSE:
+                is_binary = data[0] == '\x01'
+                num_columns = _bytes_to_bint(data[1:3])
+                DEBUG_OUTPUT("COPY_OUT_RESPONSE:", is_binary, num_columns)
+            elif code == PG_COPY_DATA:
+                DEBUG_OUTPUT("COPY_DATA")
+                obj.write(data)
+            elif code == PG_COPY_DONE:
+                DEBUG_OUTPUT("COPY_DONE")
+            elif code == PG_B_COPY_IN_RESPONSE:
+                is_binary = data[0] == '\x01'
+                num_columns = _bytes_to_bint(data[1:3])
+                DEBUG_OUTPUT("COPY_IN_RESPONSE", is_binary, num_columns)
+                while True:
+                    buf = obj.read(8192)
+                    if not buf:
+                        break
+                    self._write(PG_COPY_DATA + _bint_to_bytes(len(buf) + 4, 4))
+                    self._write(buf)
+                    self._flush()
+                self._write(PG_COPY_DONE + b'\x00\x00\x00\x04' + PG_F_SYNC + b'\x00\x00\x00\x04')
+                self._flush()
             else:
                 DEBUG_OUTPUT("SKIP:", code, ln, binascii.b2a_hex(data))
-
         return
 
     def __enter__(self):
@@ -563,6 +565,28 @@ class Connection(object):
     def _flush(self):
         pass
 
+    def _open(self):
+        self.sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
+        self.sock.connect((self.host, self.port))
+        DEBUG_OUTPUT("socket %s:%d" % (self.host, self.port))
+        if self.use_ssl:
+            self._write(_bint_to_bytes(8, 4))
+            self._write(_bint_to_bytes(80877103, 4))    # SSL request
+            if self_read(1) == b'S':
+                self.sock = ssl.wrap_socket(self.sock)
+            else:
+                raise InterfaceError("Server refuses SSL")
+        self.sock.settimeout(self.timeout)
+        v = b''.join([
+            _bint_to_bytes(196608, 4),  # protocol version 3.0
+            b'user\x00', self.user.encode('ascii'), b'\x00',
+            b'database\x00', self.database.encode('ascii'), b'\x00',
+            b'\x00',
+        ])
+        self._write(_bint_to_bytes(len(v) + 4, 4) + v)
+        self._cursor = self.cursor()
+        self._process_messages()
+
     def _close(self):
         self.sock.close()
         self.sock = None
@@ -604,10 +628,15 @@ class Connection(object):
         self._process_messages(self._cursor)
         self.begin()
 
+    def reopen(self):
+        self.close()
+        self._open()
+
     def close(self):
         DEBUG_OUTPUT('Connection::close()')
-        self._write(b''.join([PG_F_TERMINATE, b'\x00\x00\x00\x04']))
-        self._close()
+        if self.sock:
+            self._write(b''.join([PG_F_TERMINATE, b'\x00\x00\x00\x04']))
+            self._close()
 
 def connect(host, user, password, database, port=5432, timeout=60, use_ssl=False):
     return Connection(user, password, database, host, port, timeout, use_ssl)
