@@ -26,6 +26,7 @@
 import minipg
 import unittest
 import minipg
+import io
 
 class TestMiniPG(unittest.TestCase):
     host='localhost'
@@ -167,6 +168,51 @@ class TestMiniPG(unittest.TestCase):
         cur.execute("select b from test_binary")
         self.assertEqual(cur.fetchone()[0], data)
 
+    def test_copy(self):
+        cur = self.connection.cursor()
+        try:
+            cur.execute("drop table test_copy")
+        except:
+            pass
+
+        cur.execute("""
+            create table test_copy (
+              pk        serial,
+              b1        boolean,
+              i2        smallint,
+              s         varchar(255)
+            )
+        """)
+        cur.execute(u"""
+            insert into test_copy (b1, i2 ,s) values
+                (TRUE, 1, 'あいうえお'),
+                (FALSE, 2, 'かきくけこ'),
+                (FALSE, 3, 'ABC''s')
+        """)
+        self.connection.commit()
+
+        # COPY TO
+        self.connection.close()
+        self.connection.reopen()
+        text = b"1\tt\t1\t\xe3\x81\x82\xe3\x81\x84\xe3\x81\x86\xe3\x81\x88\xe3\x81\x8a\n2\tf\t2\t\xe3\x81\x8b\xe3\x81\x8d\xe3\x81\x8f\xe3\x81\x91\xe3\x81\x93\n3\tf\t3\tABC's\n"
+        f = io.BytesIO()
+        self.connection.execute(f, u"copy test_copy to stdout")
+        self.assertEqual(text, f.getvalue())
+
+        # COPY FROM
+        cur.execute("truncate table test_copy")
+        cur.execute("select count(*) from test_copy")
+        self.assertEqual(cur.fetchone()[0], 0)
+        self.connection.execute(io.BytesIO(text), u"copy test_copy from stdin")
+        self.connection.commit()
+
+        # reconnect and check
+        self.connection.close()
+        self.connection.reopen()
+        f = io.BytesIO()
+        self.connection.execute(f, u"copy test_copy to stdout")
+        self.assertEqual(text, f.getvalue())
+
 if __name__ == "__main__":
     import unittest
     unittest.main()
